<?php declare(strict_types=1);

namespace Expedico;

use Closure;

require_once __DIR__ . '/vendor/autoload.php';

/**
 * Plugin Name:         Expedico
 * Description:         Single point for your ecommerce parcel delivery, returns handling and fulfillment service mainly in Eastern Europe.
 * Version:             1.0.0
 * Author:              Expedico
 * Author URI:          https://expedico.eu
 * License:             GPL-2.0
 * License URI:         https://www.gnu.org/licenses/gpl-2.0.html
 * Text Domain:         expedico
 * WordPress Available: yes
 * Requires License:    no
 * Text Domain:         expedico
 * Domain Path:         /languages
 * Requires PHP:        7.4
 */

if (!defined('ABSPATH')) {
    exit;
}

final class Expedico
{

    public const DOMAIN = 'expedico';

    private const USERNAME    = 'wp_expedico_username';
    private const PASSWORD    = 'wp_expedico_password';
    private const BASE_URL    = 'wp_expedico_base_url';
    private const FORMAT      = 'wp_expedico_format';
    private const CARRIER_IDS = 'wp_expedico_carrier_ids';

    private string $action = 'expedico_download_labels';

    public function __construct()
    {
        add_action('init', [$this, 'loadTranslation']);

        add_action('woocommerce_init', function (): void {
            add_filter(
                'bulk_actions-edit-shop_order',
                function (array $actions): array {
                    $actions[$this->action] = 'Download labels';

                    return $actions;
                },
                100,
                1,
            );

            add_filter(
                'handle_bulk_actions-edit-shop_order',
                function (string $redirectUrl, string $action, array $orderIds): string {
                    if ($action !== $this->action) {
                        return $redirectUrl;
                    }

                    $parcelIds = $this->getParcelIds($orderIds);

                    if (!$parcelIds) {
                        return $redirectUrl;
                    }

                    $labels = $this->getLabels($parcelIds);

                    if (!$labels) {
                        return add_query_arg([$this->action => TRUE], $redirectUrl);
                    }

                    header('Content-type: application/pdf');
                    header('Content-Disposition: attachment; filename=labels.pdf');
                    echo $labels;

                    exit(0);
                },
                100,
                3
            );
        });

        add_filter('woocommerce_shipping_methods', [$this, 'add_shipping_method']);

        add_action('admin_notices', function (): void {
            if (!isset($_REQUEST[$this->action])) {
                return;
            }

            printf('<div class="notice notice-error"><p>Something gone wrong when calling Expedico.eu API!</p></div>');
        });

        add_action('rest_api_init', function (): void {
            $this->createRoute(
                'GET',
                '/settings',
                fn(): array => [
                    'username'    => $this->getSettings(self::USERNAME),
                    'password'    => $this->getSettings(self::PASSWORD),
                    'base_url'    => $this->getSettings(self::BASE_URL),
                    'format'      => $this->getSettings(self::FORMAT),
                    'carrier_ids' => json_decode($this->getSettings(self::CARRIER_IDS)),
                ]
            );

            $this->createRoute(
                'POST',
                '/settings',
                fn(\WP_REST_Request $request): array => [
                    'username'    => $this->setSettings(self::USERNAME, $request->get_json_params()['username']),
                    'password'    => $this->setSettings(self::PASSWORD, $request->get_json_params()['password']),
                    'base_url'    => $this->setSettings(self::BASE_URL, $request->get_json_params()['base_url']),
                    'format'      => $this->setSettings(self::FORMAT, $request->get_json_params()['format']),
                    'carrier_ids' => $this->setSettings(
                        self::CARRIER_IDS,
                        json_encode($request->get_json_params()['carrier_ids']),
                    ),
                ],
                [
                    'username'    => [
                        'required' => TRUE,
                    ],
                    'password'    => [
                        'required' => TRUE,
                    ],
                    'base_url'    => [
                        'required' => TRUE,
                    ],
                    'format'      => [
                        'required' => TRUE,
                    ],
                    'carrier_ids' => [
                        'required' => TRUE,
                    ],
                ]
            );
        });

        add_action('woocommerce_after_shipping_rate', [$this, 'renderWidgetButtonTableRow'], 1, 2);
        add_action('woocommerce_checkout_process', [$this, 'validateCheckoutData']);
        add_action('woocommerce_checkout_update_order_meta', [$this, 'updateOrderMetadata']);

        register_uninstall_hook(__FILE__, [__CLASS__, 'uninstall']);
    }

    public function validateCheckoutData(): void
    {
        if (!array_key_exists('pickupId', $_COOKIE)) {
            wc_add_notice(__('Pickup point is not selected.', 'expedico'), 'error');
        }
    }

    public function updateOrderMetadata($orderId): void
    {
        update_post_meta($orderId, 'expedico_pickup_id', $_COOKIE['pickupId']);
        setcookie("pickupId", "", time() - 3600);
    }

    public function renderWidgetButtonTableRow(\WC_Shipping_Rate $shippingRate, int $index): void
    {
        $method = WC()->session->get('chosen_shipping_methods')[$index];

        if (!$this->isExpedicoShippingMethod($method) || $method !== $shippingRate->get_id()) {
            return;
        }

        if (!is_checkout()) {
            return;
        }

        setcookie("pickupId", "", time() - 3600);

        $carriers = json_decode(str_replace('\\', '', $this->getSettings(self::CARRIER_IDS)), TRUE);

        echo str_replace(
            ['optionsPlaceholder', 'choosePickupPlaceholder'],
            [
                json_encode(
                    [
                        "carrier_ids" => [intval($carriers[$shippingRate->instance_id] ?? 0)],
                    ]
                ), __('Choose pick up point', 'expedico'),
            ],
            file_get_contents(__DIR__ . "/choose-pickup.html")
        );
    }

    /**
     * @param array $methods Previous state.
     *
     * @return array
     */
    public function add_shipping_method(array $methods): array
    {
        $methods[ShippingMethod::EXPEDICO_METHOD_ID] = ShippingMethod::class;

        return $methods;
    }

    private static function isWooCommercePluginActive(): bool
    {
        $pluginRelativePath = 'woocommerce/woocommerce.php';
        if (is_multisite()) {
            $plugins = get_site_option('active_sitewide_plugins');
            if (isset($plugins[$pluginRelativePath])) {
                return TRUE;
            }
        }

        if (!function_exists('get_mu_plugins')) {
            require_once ABSPATH . 'wp-admin/includes/plugin.php';
        }

        $muPlugins = get_mu_plugins();
        if (isset($muPlugins[$pluginRelativePath])) {
            return TRUE;
        }

        return in_array($pluginRelativePath, (array) get_option('active_plugins', []), TRUE);
    }

    private function isExpedicoShippingMethod(string $chosenMethod): bool
    {
        return strpos($chosenMethod, ShippingMethod::EXPEDICO_METHOD_ID) !== FALSE;
    }

    public static function getLocale(): string
    {
        return (string) apply_filters(
            'plugin_locale',
            (is_admin() ? get_user_locale() : get_locale()),
            self::DOMAIN
        );
    }

    public function loadTranslation(): void
    {
        $domain = self::DOMAIN;
        $locale = self::getLocale();

        $moFile = __DIR__ . "/languages/$domain-$locale.mo";
        $result = load_textdomain($domain, $moFile);

        if (FALSE === $result) {
            $moFile = __DIR__ . "/languages/$domain-en_US.mo";
            load_textdomain($domain, $moFile);
        }
    }

    public static function uninstall(): void
    {
        delete_option(self::USERNAME);
        delete_option(self::PASSWORD);
        delete_option(self::BASE_URL);
        delete_option(self::FORMAT);
        delete_option(self::CARRIER_IDS);
    }

    private function getSettings(string $name): string
    {
        return get_option($name, '');
    }

    private function setSettings(string $name, string $value): string
    {
        update_option($name, esc_sql($value));

        return $this->getSettings($name);
    }

    private function getParcelIds(array $orderIds): array
    {
        $parcelIds = [];

        foreach ($orderIds as $orderId) {
            $notes = wc_get_order_notes(['order_id' => $orderId, 'order_by' => 'date_created', 'type' => 'internal']);

            foreach ($notes as $note) {
                if (strpos($note->content, 'Expedico: {ParcelId:') === 0) {
                    $parcelIds[] = trim(trim(explode(':', $note->content)[2]), '}');
                }
            }
        }

        return $parcelIds;
    }

    private function getLabels(array $parcelIds): ?string
    {
        $url = json_decode(
                   wp_remote_retrieve_body(
                       wp_remote_post(sprintf('%s/label_collections', $this->getSettings(self::BASE_URL)), [
                           'headers' => [
                               'Authorization' => sprintf(
                                   'Basic %s',
                                   base64_encode(
                                       sprintf(
                                           '%s:%s',
                                           $this->getSettings(self::USERNAME),
                                           $this->getSettings(self::PASSWORD),
                                       )
                                   )
                               ),
                           ],
                           'body'    => [
                               'data' => [
                                   'parcel_ids' => implode(',', $parcelIds),
                                   'format'     => $this->getSettings(self::FORMAT),
                               ],
                           ],
                       ])
                   ),
                   TRUE
               )['data']['attributes']['labels_url'];

        if (!$url) {
            return NULL;
        }

        return wp_remote_retrieve_body(wp_remote_get($url));
    }

    private function createRoute(string $method, string $url, Closure $callback, array $arguments = []): void
    {
        register_rest_route('expedico', $url, [
            'methods'             => $method,
            'callback'            => $callback,
            'args'                => $arguments,
            'permission_callback' => fn() => current_user_can('manage_options'),
        ]);
    }
}

add_action('plugins_loaded', fn(): Expedico => new Expedico());
